<?php
namespace Ilios\CliBundle\Tests\Command;

use Ilios\CliBundle\Command\FindUserCommand;
use Symfony\Component\Console\Application;
use Symfony\Component\Console\Tester\CommandTester;
use Mockery as m;

class FindUserCommandTest extends \PHPUnit_Framework_TestCase
{
    const COMMAND_NAME = 'ilios:directory:find-user';
    
    protected $commandTester;
    protected $directory;
    
    public function setUp()
    {
        $this->directory = m::mock('Ilios\CoreBundle\Service\Directory');
        $command = new FindUserCommand($this->directory);
        $application = new Application();
        $application->add($command);
        $commandInApp = $application->find(self::COMMAND_NAME);
        $this->commandTester = new CommandTester($commandInApp);
    }

    /**
     * Remove all mock objects
     */
    public function tearDown()
    {
        unset($this->directory);
        unset($this->commandTester);
        m::close();
    }
    
    public function testExecute()
    {
        $fakeDirectoryUser = [
            'firstName' => 'first',
            'lastName' => 'last',
            'email' => 'email',
            'telephoneNumber' => 'phone',
            'campusId' => 'abc',
        ];
        $this->directory->shouldReceive('find')->with(array('a', 'b'))->andReturn(array($fakeDirectoryUser));
        
        $this->commandTester->execute(array(
            'command'      => self::COMMAND_NAME,
            'searchTerms'         => array('a', 'b')
        ));
        
        
        $output = $this->commandTester->getDisplay();
        $this->assertRegExp(
            '/abc       | first | last | email | phone/',
            $output
        );
    }
    
    public function testTermRequired()
    {
        $this->setExpectedException('RuntimeException');
        $this->commandTester->execute(array('command' => self::COMMAND_NAME));
    }
}
